// -*- C++ -*-
/*!
 * @file  EV3Control.cpp
 * @brief EV3Mindstorms Control
 * @date $Date$
 *
 * $Id$
 */

#include "EV3Control.h"

// Module specification
// <rtc-template block="module_spec">
static const char* ev3control_spec[] =
  {
	"implementation_id", "EV3Control",
	"type_name",		 "EV3Control",
	"description",		 "EV3Mindstorms Control",
	"version",			 "1.0.0",
	"vendor",			 "Aizu",
	"category", 		 "EV3",
	"activity_type",	 "PERIODIC",
	"kind", 			 "DataFlowComponent",
	"max_instance", 	 "1",
	"language", 		 "C++",
	"lang_type",		 "compile",
	""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
EV3Control::EV3Control(RTC::Manager* manager)
	// <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
	m_VelIn("vel", m_Vel),
	m_ultrasonicOut("ultrasonic", m_ultrasonic),
	m_colorOut("color", m_color),
	m_touchOut("touch", m_touch),
	color(INPUT_1),
	touch1(INPUT_2),
	ultrasonic(INPUT_3),
	touch2(INPUT_4),
	motorA(OUTPUT_A),
	motorB(OUTPUT_B)

	// </rtc-template>
{
}

/*!
 * @brief destructor
 */
EV3Control::~EV3Control()
{
}



RTC::ReturnCode_t EV3Control::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("vel", m_VelIn);
  
  // Set OutPort buffer
  addOutPort("ultrasonic", m_ultrasonicOut);
  addOutPort("color", m_colorOut);
  addOutPort("touch", m_touchOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  std::cout << "onInitialize" << std::endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t EV3Control::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t EV3Control::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "onActive" << std::endl;

	if (!color.connected())	std::cout << "color disconnect" << std::endl;
	if (!touch1.connected())std::cout << "touch1 disconnect" << std::endl;
	if (!touch2.connected())std::cout << "touch2 disconnect" << std::endl;
	if (!ultrasonic.connected())std::cout << "ultrasonic disconnect" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t EV3Control::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "onDeactive" << std::endl;
	motorA.stop();
	motorB.stop();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t EV3Control::onExecute(RTC::UniqueId ec_id)
{
	// J[ZT[
	stringstream ss;
	ss << color.color();
	m_color.data = ss.str().c_str();
	// J[ZT[l
	m_colorOut.write();
	std::cout << "color:" << m_color.data;
	
	// ^b`ZT[
	m_touch.data.length(2);
	m_touch.data[0] = touch1.is_pressed();
	m_touch.data[1] = touch2.is_pressed();
	// ^b`ZT[l
	m_touchOut.write();
	std::cout << " touch1:" << m_touch.data[0] << " touch2:" << m_touch.data[1];
	
	// gZT[
	m_ultrasonic.ranges.length(1);
	m_ultrasonic.ranges[0] = ultrasonic.distance_centimeters();
	// gZT[l
	m_ultrasonicOut.write();
	std::cout << " ulsonic:" << m_ultrasonic.ranges[0];
	
	// x񂪍XVĂꍇ
	if (m_VelIn.isNew())
	{
		// x擾
		m_VelIn.read();
		
		//pxvZ
		float W_l = (m_Vel.data.vx - 0.05925 * m_Vel.data.va) / 0.028;
		float W_r = (m_Vel.data.vx + 0.05925 * m_Vel.data.va) / 0.028;
		cout << " Wl:" << W_l << ",Wr:" << W_r;
		

		//xbvZ
		float sp_l = (int)(W_l * (180 / M_PI)*(motorA.count_per_rot() / (float)360.0));
		float sp_r = (int)(W_r * (180 / M_PI)*(motorB.count_per_rot() / (float)360.0));
		cout << " spl:" << sp_l;
		cout << " spr:" << sp_r;

		// lAl
		if (sp_l > 2000) sp_l = 2000;
		else if (sp_l < -2000)sp_l = -2000;
		if (sp_r > 2000) sp_r = 2000;
		else if (sp_r < -2000)sp_r = -2000;

		int desl = (int)sp_l;
		int desr = (int)sp_r;
		
		// [^Xs[hݒ
		motorA.set_speed_sp(desl);
		motorB.set_speed_sp(desr);

		//motorA.set_speed_sp(sp_l);
		//motorB.set_speed_sp(sp_r);

		// [^]
		motorA.run_forever();
		motorB.run_forever();
		
	}
	cout << " " << endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t EV3Control::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Control::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void EV3ControlInit(RTC::Manager* manager)
  {
	coil::Properties profile(ev3control_spec);
	manager->registerFactory(profile,
							 RTC::Create<EV3Control>,
							 RTC::Delete<EV3Control>);
  }
  
};


